﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Sample.Models;
using Infragistics.Web.Mvc;
using Core.Data;
using Core;
using Core.Data.BusinessObjects;

namespace Sample.Controllers
{
    public class ModelController : Controller
    {
        public ActionResult DefinedColumns()
        {
            ViewModel viewModel = new ViewModel();
            
            viewModel.MasterGrid = GetMasterGridModel();
			viewModel.DetailsGrid = GetDetailsGridModel();
            return View(viewModel);
        }

        private GridModel GetMasterGridModel()
        {
            GridModel masterGridModel = new GridModel();
            masterGridModel.ID = "masterGrid";
            masterGridModel.Caption = "Master Grid (Projects)";
            masterGridModel.AutoGenerateColumns = false;
            masterGridModel.AutoGenerateLayouts = false;
            masterGridModel.LoadOnDemand = true;
            masterGridModel.PrimaryKey = "ProjectId";

            masterGridModel.Columns = new List<GridColumn>();
            masterGridModel.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "100px"));
            masterGridModel.Columns.Add(new GridColumn("Name", "Name", "string", "150px"));
            masterGridModel.Columns.Add(new GridColumn("Start Date", "StartDate", "date", "150px"));
            masterGridModel.Columns.Add(new GridColumn("End Date", "EndDate", "date", "150px"));
            masterGridModel.Columns.Add(new GridColumn("Is Postponed", "IsPostponed", "bool", "150px"));

            masterGridModel.DataSource = GetProjectData();
            masterGridModel.DataSourceUrl = Url.Action("BindProjects");

            GridColumnLayoutModel developersModel = GetDevelopersLayout();
            GridColumnLayoutModel virtualMachinesModel = GetVirtualMachinesLayout();
            developersModel.ColumnLayouts.Add(virtualMachinesModel);
            masterGridModel.ColumnLayouts.Add(developersModel);

            GridColumnLayoutModel scrumsModel = GetScrumsLayout();
            masterGridModel.ColumnLayouts.Add(scrumsModel);

            GridColumnLayoutModel tasksModel = GetTasksLayout();
            GridColumnLayoutModel checkinsModel = GetCheckinsLayout();
            GridColumnLayoutModel filesModel = GetFilesLayout();

            checkinsModel.ColumnLayouts.Add(filesModel);
            tasksModel.ColumnLayouts.Add(checkinsModel);
            masterGridModel.ColumnLayouts.Add(tasksModel);

            GridSorting rootSorting = new GridSorting();
            rootSorting.Type = OpType.Remote;
            rootSorting.Mode = SortingMode.Multiple;
            rootSorting.Inherit = false;
            masterGridModel.Features.Add(rootSorting);

			GridSelection rootSelection = new GridSelection();
			rootSelection.Mode = SelectionMode.Row;
			masterGridModel.Features.Add(rootSelection);

            //GridPaging rootPaging = new GridPaging();
            //rootPaging.Type = OpType.Remote;
            //rootPaging.Inherit = false;
            //gridModel.Features.Add(rootPaging);

            return masterGridModel;
        }

		private GridModel GetDetailsGridModel()
		{
			GridModel detailsGridModel = new GridModel();
            detailsGridModel.ID = "detailsGrid";
            detailsGridModel.Caption = "Details Grid (Scrum meetings for the selected project)";
			detailsGridModel.AutoGenerateColumns = false;

            detailsGridModel.Columns.Add(new GridColumn("Meeting ID", "MeetingId", "number", "100px"));
            detailsGridModel.Columns.Add(new GridColumn("Project ID", "ProjectId", "string", "100px"));
            detailsGridModel.Columns.Add(new GridColumn("Occurred On", "OccurredOn", "date", "100px"));
            detailsGridModel.Columns.Add(new GridColumn("Scrum Master", "ScrumMaster", "string", "100px"));
            detailsGridModel.Columns.Add(new GridColumn("Notes", "Notes", "string", "150px"));
            detailsGridModel.Columns.Add(new GridColumn("Priority", "Priority", "number", "100px"));
            detailsGridModel.Columns.Add(new GridColumn("Are All Required?", "AreAllRequired", "bool", "100px"));

			
			DevelopmentDBEntities entities = new DevelopmentDBEntities();
			var query = from s in entities.Scrums
						where 1 == 2
						orderby s.MeetingID
						select new MyScrum
						{
							ProjectId = s.ProjectID,
							MeetingId = s.MeetingID,
							OccurredOn = s.OccurredOn,
							ScrumMaster = s.ScrumMaster,
							Notes = s.Notes,
							Priority = s.Priority,
							AreAllRequired = s.AreAllRequired
						};


			detailsGridModel.DataSource = GetScrums();
			detailsGridModel.DataSourceUrl = Url.Action("BindDetailsGrid", new { layout = "Scrums"});

			return detailsGridModel;
		}

        #region Data Access

        private IQueryable<MyProject> GetProjectData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from p in entities.Projects
                        orderby p.ProjectID
                        select new MyProject
                        {
                            ProjectId = p.ProjectID,
                            Name = p.Name,
                            StartDate = p.StartDate,
                            EndDate = p.EndDate,
                            IsPostponed = p.IsPostponed,
                            Developers = from d in p.Developers
                                         orderby d.DeveloperID
                                         select new MyDeveloper
                                         {
                                             DeveloperId = d.DeveloperID,
                                             ProjectId = d.ProjectID,
                                             Name = d.Name,
                                             Position = d.Position,
                                             HireDate = d.HireDate,
                                             Salary = d.Salary,
                                             VirtualMachines = from v in d.VirtualMachines
                                                               orderby v.VirtualMachineID
                                                               select new MyVirtualMachine
                                                               {
                                                                   VirtualMachineId = v.VirtualMachineID,
                                                                   DeveloperId = v.DeveloperID,
                                                                   OperatingSystem = v.OperatingSystem,
                                                                   DelegatedSpace = v.DelegatedSpace,
                                                                   ExpirationDate = v.ExpirationDate
                                                               }
                                         },
                            Scrums = from s in p.Scrums
                                     orderby s.MeetingID
                                     select new MyScrum
                                     {
                                         ProjectId = s.ProjectID,
                                         MeetingId = s.MeetingID,
                                         OccurredOn = s.OccurredOn,
                                         ScrumMaster = s.ScrumMaster,
                                         Notes = s.Notes,
                                         Priority = s.Priority,
                                         AreAllRequired = s.AreAllRequired
                                     },
                            Tasks = from t in p.Tasks
                                    orderby t.TaskID
                                    select new MyTask
                                    {
                                        TaskId = t.TaskID,
                                        ProjectId = t.ProjectID,
                                        Name = t.Name,
                                        ParentTaskId = t.ParentTaskID,
                                        IsCompleted = t.IsCompleted,
                                        Checkins = from c in t.Checkins
                                                   orderby c.CheckinID
                                                   select new MyCheckin
                                                   {
                                                       CheckinId = c.CheckinID,
                                                       TaskId = c.TaskID,
                                                       CheckinDate = c.CheckinDate,
                                                       Files = from f in c.Files
                                                               orderby f.FileID
                                                               select new MyFile
                                                               {
                                                                   FileId = f.FileID,
                                                                   CheckinId = f.CheckinID,
                                                                   Name = f.Name,
                                                                   Size = f.Size
                                                               }
                                                   }
                                    }
                        };
            return query;
        }

		private IQueryable<MyScrum> GetScrums()
		{
			DevelopmentDBEntities entities = new DevelopmentDBEntities();
			var query = from s in entities.Scrums
						orderby s.MeetingID
						select new MyScrum
						{
							ProjectId = s.ProjectID,
							MeetingId = s.MeetingID,
							OccurredOn = s.OccurredOn,
							ScrumMaster = s.ScrumMaster,
							Notes = s.Notes,
							Priority = s.Priority,
							AreAllRequired = s.AreAllRequired
						};

			return query;
		}

        private IEnumerable<MyDeveloper> GetDevelopersData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from d in entities.Developers
                        orderby d.DeveloperID
                        select new MyDeveloper
                        {
                            DeveloperId = d.DeveloperID,
                            ProjectId = d.ProjectID,
                            Name = d.Name,
                            Position = d.Position,
                            HireDate = d.HireDate,
                            Salary = d.Salary,
                            VirtualMachines = from v in d.VirtualMachines
                                              orderby v.VirtualMachineID
                                              select new MyVirtualMachine
                                              {
                                                  VirtualMachineId = v.VirtualMachineID,
                                                  DeveloperId = v.DeveloperID,
                                                  OperatingSystem = v.OperatingSystem,
                                                  DelegatedSpace = v.DelegatedSpace,
                                                  ExpirationDate = v.ExpirationDate
                                              }
                        };
            return query;
        }

        private IEnumerable<MyVirtualMachine> GetVirtualMachinesData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from v in entities.VirtualMachines
                        orderby v.VirtualMachineID
                        select new MyVirtualMachine
                        {
                            VirtualMachineId = v.VirtualMachineID,
                            DeveloperId = v.DeveloperID,
                            OperatingSystem = v.OperatingSystem,
                            DelegatedSpace = v.DelegatedSpace,
                            ExpirationDate = v.ExpirationDate
                        };
            return query;
        }

        private IEnumerable<MyScrum> GetScrumsData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from s in entities.Scrums
                        orderby s.MeetingID
                        select new MyScrum
                        {
                            ProjectId = s.ProjectID,
                            MeetingId = s.MeetingID,
                            OccurredOn = s.OccurredOn,
                            ScrumMaster = s.ScrumMaster,
                            Notes = s.Notes,
                            Priority = s.Priority,
                            AreAllRequired = s.AreAllRequired
                        };

            return query;
        }

        private IEnumerable<MyTask> GetTasksData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from t in entities.Tasks
                        orderby t.TaskID
                        select new MyTask
                        {
                            TaskId = t.TaskID,
                            ProjectId = t.ProjectID,
                            Name = t.Name,
                            ParentTaskId = t.ParentTaskID,
                            IsCompleted = t.IsCompleted
                        };
            return query;
        }

        private IEnumerable<MyCheckin> GetCheckinsData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from c in entities.Checkins
                        orderby c.CheckinID
                        select new MyCheckin
                        {
                            CheckinId = c.CheckinID,
                            TaskId = c.TaskID,
                            CheckinDate = c.CheckinDate
                        };
            return query;                                       
        }

        private IEnumerable<MyFile> GetFilesData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from f in entities.Files
                        orderby f.FileID
                        select new MyFile
                        {
                            FileId = f.FileID,
                            CheckinId = f.CheckinID,
                            Name = f.Name,
                            Size = f.Size
                        };
            return query;
        }

        #endregion // Data Access

        #region DataSourceUrls

        public JsonResult BindAutoGeneratedProjects()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = true;
            gridModel.AutoGenerateLayouts = true;
            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindAutoGeneratedProjects");
            gridModel.LoadOnDemand = false;

            ViewModel viewModel = new ViewModel();

            GridSorting rootSorting = new GridSorting();
            rootSorting.Type = OpType.Remote;
            rootSorting.Mode = SortingMode.Multiple;
            rootSorting.Inherit = false;
            gridModel.Features.Add(rootSorting);

            GridPaging rootPaging = new GridPaging();
            rootPaging.Type = OpType.Remote;
            rootPaging.Inherit = false;
            gridModel.Features.Add(rootPaging);

            viewModel.MasterGrid = gridModel;
            return gridModel.GetData();
        }

		public JsonResult BindDetailsGrid(string path, string layout)
		{
			GridModel gridModel =  GetMasterGridModel();
			gridModel.AutoGenerateColumns = true;
			gridModel.DataSource = GetScrums();
			
			return gridModel.GetData(path, layout);
		}

        public JsonResult BindProjects()
        {
            GridModel gridModel = GetMasterGridModel();
            return gridModel.GetData();
        }

        public JsonResult BindDevelopers(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetDevelopersData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindScrums(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetScrumsData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindTasks(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetTasksData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindCheckins(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetCheckinsData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindFiles(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetFilesData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindVirtualMachines(string path, string layout)
        {
            GridModel gridModel = GetMasterGridModel();
            gridModel.DataSource = GetVirtualMachinesData();
            return gridModel.GetData(path, layout);
        }

        #endregion // DataSourceUrls

        #region Layouts

        private GridColumnLayoutModel GetDevelopersLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Developers";
            layout.PrimaryKey = "DeveloperId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Developers";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Developer Id", "DeveloperId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Positon", "Position", "string", "200px"));
            layout.Columns.Add(new GridColumn("Hire Date", "HireDate", "date", "200px"));
            layout.Columns.Add(new GridColumn("Salary", "Salary", "number", "200px"));

            layout.DataSourceUrl = Url.Action("BindDevelopers");

            return layout;
        }

        private GridColumnLayoutModel GetVirtualMachinesLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "VirtualMachines";
            layout.PrimaryKey = "VirtualMachineId";
            layout.ForeignKey = "DeveloperId";
            layout.Caption = "Virtual Machines";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Virtual Machine Id", "VirtualMachineId", "string", "200px"));
            layout.Columns.Add(new GridColumn("Developer Id", "DeveloperId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Operating System", "OperatingSystem", "string", "200px"));
            layout.Columns.Add(new GridColumn("Delegated Space", "DelegatedSpace", "number", "200px"));
            layout.Columns.Add(new GridColumn("Expiration Date", "ExpirationDate", "date", "200px"));

            layout.DataSourceUrl = Url.Action("BindVirtualMachines");

            return layout;
        }

        private GridColumnLayoutModel GetScrumsLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Scrums";
            layout.PrimaryKey = "MeetingId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Scrums";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Meeting Id", "MeetingId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Occurred On", "OccurredOn", "date", "200px"));
            layout.Columns.Add(new GridColumn("Scrum Master", "ScrumMaster", "string", "200px"));
            layout.Columns.Add(new GridColumn("Notes", "Notes", "string", "200px"));
            layout.Columns.Add(new GridColumn("Priority", "Priority", "number", "200px"));
            layout.Columns.Add(new GridColumn("Are All Required", "AreAllRequired", "bool", "200px"));

            layout.DataSourceUrl = Url.Action("BindScrums");

            return layout;
        }

        private GridColumnLayoutModel GetTasksLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Tasks";
            layout.PrimaryKey = "TaskId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Tasks";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Task Id", "TaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Parent Task Id", "ParentTaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Is Completed", "IsCompleted", "bool", "200px"));

            layout.DataSourceUrl = Url.Action("BindTasks");

            return layout;
        }

        private GridColumnLayoutModel GetCheckinsLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Checkins";
            layout.PrimaryKey = "CheckinId";
            layout.ForeignKey = "TaskId";
            layout.Caption = "CheckIns";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Checkin Id", "CheckinId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Task Id", "TaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Checkin Date", "CheckinDate", "date", "200px"));

            layout.DataSourceUrl = Url.Action("BindCheckins");

            return layout;
        }

        private GridColumnLayoutModel GetFilesLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Files";
            layout.PrimaryKey = "FileId";
            layout.ForeignKey = "CheckinId";
            layout.Caption = "Files";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Files Id", "FileId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Checkin Id", "CheckinId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Size", "Size", "number", "200px"));

            layout.DataSourceUrl = Url.Action("BindFiles");

            return layout;
        }

        #endregion // Layouts


    }
}
